==> adlib-2.2.0.tgz: package/lib/adlib.js <==
/**
 * adlib.js
 */
import getWithDefault from './getWithDefault';
import deepMapValues from './deepMap';
import {arborist} from './optional-transform/arborist';
import optionalTransform from './optional-transform/optional';

function isString(v) {
  return typeof v === 'string';
}

function _swap(parameter, settings, transforms) {
  let value;
  // console.info(`_swap: param: ${parameter}`);
  // Parameters can optionally call transform functions
  // e.g. "{{ipsum:translateLatin}}"
  // so extract {{<parameter>:<transformFunction>:<key||value>}}
  let transformCheck = parameter.split(':');
  if (transformCheck.length > 1) {
    // we have a request to use a transform...
    let key = transformCheck[0];
    let fn = transformCheck[1];
    // we default to using the value...
    let param = null;
    if (transformCheck[2]){
      param = transformCheck[2];
    }
    if(transforms && transforms[fn] && typeof transforms[fn] === 'function') {
      // get the value from the param
      value = getWithDefault(settings, key);
      // transform it...
      value = transforms[fn](key, value, settings, param);
    } else {
      throw new Error(`Attempted to apply non-existant transform ${fn} on ${key} with params ${parameter}`);
    }
  } else {
    // we just get the value
    value = getWithDefault(settings, parameter);
  }
  return value;
}

/**
 * Does a propertyPath exist on a target
 */
function _propertyPathExists (propertyPath, target) {
  // remove any transforms
  let cleanPath = propertyPath.split(':')[0];
  let value = getWithDefault(target, cleanPath, null);
  if (value !== null && value !== undefined) {
    return true;
  } else {
    return false;
  }
}

// Combine a Template with Settings
export default function adlib(template, settings, transforms = null) {
  transforms = transforms || {};
  if (transforms.optional) {
    throw new Error('Please do not pass in an `optional` transform, adlib provides that interally.');
  } else {
    transforms.optional = optionalTransform;
  }

  let res = deepMapValues(template, function(templateValue, templatePath){
    // Only string templates
    if (!isString(templateValue)) {
      return templateValue;
    }

    // When we match "{{layer.fields..}}"
    var settingsValue;
    var replaceValue = false;

    var handlebars = /{{[\S\s]*?}}/g;
    let hbsEntries = templateValue.match(handlebars);

    if (hbsEntries && hbsEntries.length) {
      // console.log(`Got a ${hbsEntries.length} handlebar entries...`);
      let isStaticValue = false;
      // iterate over the entries...
      let values = hbsEntries.map((entry) => {
        // console.info(`Matched ${entry}...`);
        // strip off the curlies...
        let path = entry.replace(/{|}/g, '');
        // check for || which indicate a hiearchy
        if (path.indexOf('||') > -1) {
          var paths = path.split('||');
          let numberOfPaths = paths.length;
          // here we check each option, in order, and return the first with a value in the hash, OR the last
          path = paths.find((pathOption, idx) => {
            // console.info(`Checking to see if ${pathOption} is in settings hash...`);
            let exists = _propertyPathExists(pathOption, settings);
            if (!exists) {
              if ((idx + 1) === numberOfPaths) {
                // console.info(`Got to last entry, and still did not find anything... assuming ${pathOption} is a static value...`);
                isStaticValue = true;
                // check if we can convert this into a number...
                if (!isNaN(pathOption)) {
                  pathOption = parseInt(pathOption);
                }
                return pathOption;
              } else {
                return false;
              }
            } else {
              return pathOption;
            }
          });
        }
        // setup the return value...
        let result = {
          key: entry,
          value: path
        }
        // if we have a valid object path, value comes from _swap
        if (!isStaticValue) {
          result.value = _swap(path, settings, transforms) || entry;
        }
        // console.info(`Value: ${JSON.stringify(result)}`);
        return result;
      });

      values.forEach((v) => {
        // console.log(`Comparing ${templateValue} with ${v.key}`)
        if (templateValue === v.key) {
          // console.log(`template matches key, returning ${v.value}`);
          // if the value is a string...
          if (typeof v.value === 'string') {
            // and it's numeric-ish
            if(!isNaN(v.value)) {
              // and has a . in it...
              if (v.value.indexOf('.') > -1) {
                // parse as a float...
                v.value = parseFloat(v.value);
              } else {
                // parse as an int
                v.value = parseInt(v.value);
              }
            }
          }
          settingsValue = v.value;
        } else {
          // a little extra regex dance to match the '||' because '|'
          // is a Very Special Regex Character and we need to super
          // escape them for the regex to work
          // console.log(`KEY ${v.key}`);
          // console.log(`TEMPLATE ${templateValue}`);
          templateValue = templateValue.replace(v.key, v.value);
          // console.log(`template did not match key, interpolating value ${v.value} into template to produce ${templateValue}`);
        }
      });

      // if we have a value, let's return that...
      if (settingsValue) {
        // console.log(`We found a value so we return it ${settingsValue}`);
        return settingsValue;
      } else {
        // console.log(`We did not find a value so we return the template ${templateValue}`);
        // but if we don't, lets return the template itself
        return templateValue;
      }
    } else {
      // console.log(`We did not find a hbs match, so we return the template ${templateValue}`);
      // no match, return the templateValue...
      return templateValue;
    }
  });
  return arborist(res);
}

==> adlib-2.2.3.tgz: package/lib/adlib.js <==
/**
 * adlib.js
 */
import getWithDefault from './getWithDefault';
import deepMapValues from './deepMap';
import {arborist} from './optional-transform/arborist';
import optionalTransform from './optional-transform/optional';

function isString(v) {
  return typeof v === 'string';
}

function _swap(parameter, settings, transforms) {
  let value;
  // console.info(`_swap: param: ${parameter}`);
  // Parameters can optionally call transform functions
  // e.g. "{{ipsum:translateLatin}}"
  // so extract {{<parameter>:<transformFunction>:<key||value>}}
  let transformCheck = parameter.split(':');
  if (transformCheck.length > 1) {
    // we have a request to use a transform...
    let key = transformCheck[0];
    let fn = transformCheck[1];
    // we default to using the value...
    let param = null;
    if (transformCheck[2]){
      param = transformCheck[2];
    }
    if(transforms && transforms[fn] && typeof transforms[fn] === 'function') {
      // get the value from the param
      value = getWithDefault(settings, key);
      // transform it...
      value = transforms[fn](key, value, settings, param);
    } else {
      throw new Error(`Attempted to apply non-existant transform ${fn} on ${key} with params ${parameter}`);
    }
  } else {
    // we just get the value
    value = getWithDefault(settings, parameter);
  }
  return value;
}

/**
 * Does a propertyPath exist on a target
 */
function _propertyPathExists (propertyPath, target) {
  // remove any transforms
  let cleanPath = propertyPath.split(':')[0];
  let value = getWithDefault(target, cleanPath, null);
  if (value !== null && value !== undefined) {
    return true;
  } else {
    return false;
  }
}

// Combine a Template with Settings
export default function adlib(template, settings, transforms = null) {
  transforms = transforms || {};
  if (transforms.optional) {
    throw new Error('Please do not pass in an `optional` transform, adlib provides that interally.');
  } else {
    transforms.optional = optionalTransform;
  }

  let res = deepMapValues(template, function(templateValue, templatePath){
    // Only string templates
    if (!isString(templateValue)) {
      return templateValue;
    }

    // When we match "{{layer.fields..}}"
    var settingsValue;
    var replaceValue = false;

    var handlebars = /{{[\w\.\:||&\/?=]*?}}/g;
    let hbsEntries = templateValue.match(handlebars);

    if (hbsEntries && hbsEntries.length) {
      // console.log(`Got a ${hbsEntries.length} handlebar entries...`);
      let isStaticValue = false;
      // iterate over the entries...
      let values = hbsEntries.map((entry) => {
        // console.info(`Matched ${entry}...`);
        // strip off the curlies...
        let path = entry.replace(/{|}/g, '');
        // check for || which indicate a hiearchy
        if (path.indexOf('||') > -1) {
          var paths = path.split('||');
          let numberOfPaths = paths.length;
          // here we check each option, in order, and return the first with a value in the hash, OR the last
          path = paths.find((pathOption, idx) => {
            // console.info(`Checking to see if ${pathOption} is in settings hash...`);
            let exists = _propertyPathExists(pathOption, settings);
            if (!exists) {
              if ((idx + 1) === numberOfPaths) {
                // console.info(`Got to last entry, and still did not find anything... assuming ${pathOption} is a static value...`);
                isStaticValue = true;
                // check if we can convert this into a number...
                if (!isNaN(pathOption)) {
                  pathOption = parseInt(pathOption);
                }
                return pathOption;
              } else {
                return false;
              }
            } else {
              return pathOption;
            }
          });
        }
        // setup the return value...
        let result = {
          key: entry,
          value: path
        }
        // if we have a valid object path, value comes from _swap
        if (!isStaticValue) {
          result.value = _swap(path, settings, transforms) || entry;
        }
        // console.info(`Value: ${JSON.stringify(result)}`);
        return result;
      });

      values.forEach((v) => {
        // console.log(`Comparing ${templateValue} with ${v.key}`)
        if (templateValue === v.key) {
          // console.log(`template matches key, returning ${v.value}`);
          // if the value is a string...
          if (typeof v.value === 'string') {
            // and it's numeric-ish
            if(!isNaN(v.value)) {
              // and has a . in it...
              if (v.value.indexOf('.') > -1) {
                // parse as a float...
                v.value = parseFloat(v.value);
              } else {
                // parse as an int
                v.value = parseInt(v.value);
              }
            }
          }
          settingsValue = v.value;
        } else {
          // a little extra regex dance to match the '||' because '|'
          // is a Very Special Regex Character and we need to super
          // escape them for the regex to work
          // console.log(`KEY ${v.key}`);
          // console.log(`TEMPLATE ${templateValue}`);
          templateValue = templateValue.replace(v.key, v.value);
          // console.log(`template did not match key, interpolating value ${v.value} into template to produce ${templateValue}`);
        }
      });

      // if we have a value, let's return that...
      if (settingsValue) {
        // console.log(`We found a value so we return it ${settingsValue}`);
        return settingsValue;
      } else {
        // console.log(`We did not find a value so we return the template ${templateValue}`);
        // but if we don't, lets return the template itself
        return templateValue;
      }
    } else {
      // console.log(`We did not find a hbs match, so we return the template ${templateValue}`);
      // no match, return the templateValue...
      return templateValue;
    }
  });
  return arborist(res);
}
